const std    = @import("std");
const dvui   = @import("dvui");
const hncore = @import("hncore");

const compo = @import("./compo/root.zig");

const Self = @This();

stage: compo.stage.Mock,
search: compo.search.Mock,
taskedit: compo.taskedit.Mock,

pub fn init(alloc: std.mem.Allocator) !Self {
    return Self {
        .search   = try compo.search.Mock.init(alloc),
        .stage    = try compo.stage.Mock.init(alloc),
        .taskedit = try compo.taskedit.Mock.init(alloc),
    };
}
pub fn deinit(self: *Self) void {
    self.taskedit.deinit();
    self.stage.deinit();
    self.search.deinit();
}

pub fn gui(self: *Self) !void {
    try compo.menu.gui(.{});

    // background
    {
        var box = try dvui.box(@src(), .vertical, .{
            .expand = .both,
            .background = true,
            .color_fill = .{ .color = dvui.Color.white, },
        });
        defer box.deinit();
    }

    // ---- windows
    try compo.search.gui(&self.search);
    try compo.stage.gui(&self.stage);
    try compo.taskedit.gui(&self.taskedit);
}
