const std  = @import("std");
const dvui = @import("dvui");

const ui    = @import("../ui/root.zig");
const compo = @import("./root.zig");

pub const Task = struct {
    id: usize,
    summary: [:0]const u8,

    done    : bool = false,
    staged  : bool = false,
    archived: bool = false,
};

pub fn gui(ctx: anytype) !void {
    var win = try dvui.floatingWindow(@src(), .{}, .{});
    defer win.deinit();

    try dvui.windowHeader("Search", "", null);

    // task adder
    {
        var adder = try dvui.textEntry(@src(), .{}, .{
            .expand = .horizontal,
        });
        defer adder.deinit();

        if (dvui.focusedWidgetId() == adder.data().id) {
            if (ui.event.keyPress("activate")) {
                std.debug.print("hello\n", .{});
            }
        }
    }

    // task list
    {
        var scrollArea = try dvui.scrollArea(@src(), .{}, .{
            .expand = .both,
        });
        defer scrollArea.deinit();

        var list = try dvui.box(@src(), .vertical, .{
            .expand  = .both,
            .padding = dvui.Rect.all(4),
        });
        defer list.deinit();

        for (0.., ctx.tasks()) |idx, task| {
            if (idx > 0) {
                _ = try dvui.spacer(@src(), .{ .w = 0, .h = 4}, .{
                    .id_extra = idx,
                    .expand   = .horizontal,
                });
            }

            var subctx = TaskCardCtx(@TypeOf(ctx)) { .ctx = ctx, .task = task };
            try compo.taskcard.gui(@src(), &subctx);
        }

        try dvui.labelNoFmt(@src(), "no tasks anymore :)", .{ .gravity_x = 0.5 });
    }
}

fn TaskCardCtx(T: type) type {
    return struct {
        const Self = @This();

        ctx : T,
        task: Task,

        pub fn open(self: *Self) void {
            _ = self;
            std.debug.print("OPEN\n", .{});
        }

        pub fn id(self: *const Self) usize {
            return self.task.id;
        }
        pub fn summary(self: *const Self) []const u8 {
            return self.task.summary;
        }
        pub fn done(self: *const Self) bool {
            return self.task.done;
        }
        pub fn archived(self: *const Self) bool {
            return self.task.archived;
        }
        pub fn staged(self: *const Self) bool {
            return self.task.staged;
        }
    };
}

pub const Mock = struct {
    const Self     = @This();
    const TaskList = std.ArrayList(Task);

    _tasks: TaskList,

    pub fn init(alloc: std.mem.Allocator) !Self {
        var ts = TaskList.init(alloc);
        errdefer ts.deinit();

        try ts.append(Task { .id = 0, .summary = "helloworld", });
        try ts.append(Task { .id = 1, .summary = "goodbye", });

        return Mock {
            ._tasks = ts,
        };
    }
    pub fn deinit(self: *Self) void {
        self._tasks.deinit();
    }

    fn tasks(self: *const Self) []const Task {
        return self._tasks.items;
    }
};
