const std  = @import("std");
const dvui = @import("dvui");

const compo = @import("./root.zig");

pub const Task = struct {
    id: usize,
    summary: [:0]const u8,
    done: bool,
};

pub fn gui(ctx: anytype) !void {
    var win = try dvui.floatingWindow(@src(), .{}, .{});
    defer win.deinit();

    try dvui.windowHeader("Stage", "", null);

    var scrollArea = try dvui.scrollArea(@src(), .{}, .{
        .expand = .both,
    });
    defer scrollArea.deinit();

    var reorder = try dvui.reorder(@src(), .{
        .background = true,
        .expand     = .both,
    });
    defer reorder.deinit();

    var list = try dvui.box(@src(), .vertical, .{
        .expand  = .both,
        .padding = dvui.Rect.all(4),
    });
    defer list.deinit();

    const tasks = ctx.get_tasks();
    for (0.., tasks) |idx, task| {
        if (idx > 0) {
            _ = try dvui.spacer(@src(), .{ .w = 0, .h = 4}, .{
                .id_extra = idx,
                .expand   = .horizontal,
            });
        }

        var reorderable = try reorder.reorderable(@src(), .{
            .draw_target = false,
        }, .{
            .id_extra = idx,
            .expand   = .horizontal,
        });
        defer reorderable.deinit();

        var subctx = TaskCardCtx(@TypeOf(ctx)) {
            .ctx         = ctx,
            .task        = task,
            .reorderable = reorderable,
        };
        try compo.taskcard.gui(@src(), &subctx);
    }
}

fn TaskCardCtx(T: type) type {
    return struct {
        const Self = @This();

        ctx : T,
        task: Task,
        reorderable: *dvui.Reorderable,

        pub fn open(self: *Self) void {
            _ = self;
            std.debug.print("OPEN\n", .{});
        }
        pub fn startDragging(self: *Self, p: dvui.Point) void {
            self.reorderable.reorder.dragStart(self.reorderable.data().id, p);
        }

        pub fn id(self: *const Self) usize {
            return self.task.id;
        }
        pub fn summary(self: *const Self) []const u8 {
            return self.task.summary;
        }
        pub fn done(self: *const Self) bool {
            return self.task.done;
        }
        pub fn staged(_: *const Self) bool {
            return true;
        }
        pub fn archived(_: *const Self) bool {
            return false;
        }
    };
}

pub const Mock = struct {
    const Self     = @This();
    const TaskList = std.ArrayList(Task);

    tasks: TaskList,

    pub fn init(alloc: std.mem.Allocator) !Self {
        var tasks = TaskList.init(alloc);
        errdefer tasks.deinit();

        for (0..100) |id| {
            try tasks.append(Task { .id = id*2, .summary = "helloworld", .done = true, });
            try tasks.append(Task { .id = id*2+1, .summary = "goodbye", .done = false, });
        }

        return Mock {
            .tasks = tasks,
        };
    }
    pub fn deinit(self: *Self) void {
        self.tasks.deinit();
    }

    pub fn mark(self: *Self, idx: usize, check: bool) void {
        self.tasks.items[idx].done = check;
    }
    pub fn get_tasks(self: *const Self) []const Task {
        return self.tasks.items;
    }
};
