const std  = @import("std");
const dvui = @import("dvui");

const ui = @import("../ui/root.zig");

pub const Tab = enum {
    info,
    detail,
    network,
    gantt,
};

pub fn gui(ctx: anytype) !void {
    var win = try dvui.floatingWindow(@src(), .{}, .{});
    defer win.deinit();

    // window header
    {
        var buf: [128]u8 = undefined;
        const subtitle = std.fmt.bufPrint(&buf, "#{d}", .{ctx.taskId()}) catch unreachable;
        try dvui.windowHeader("Task", subtitle, null);
    }

    // task summary
    {
        var hbox = try dvui.box(@src(), .horizontal, .{ .expand = .horizontal, });
        defer hbox.deinit();

        var mark: bool = false;
        if (try dvui.checkbox(@src(), &mark, null, .{ .gravity_y = 0.5, })) {
        }

        var summary = try dvui.textEntry(@src(), .{}, .{
            .expand = .horizontal,
        });
        defer summary.deinit();
    }

    // tabs
    {
        const shown = ctx.shownTab();

        var tbox = try dvui.box(@src(), .vertical, .{ .expand = .both, });
        defer tbox.deinit();

        {
            var tabs = dvui.TabsWidget.init(@src(), .{ .dir = .horizontal }, .{
                .expand = .horizontal,
            });
            try tabs.install();
            defer tabs.deinit();

            inline for (std.meta.fields(Tab)) |tabMeta| {
                const tab = @field(Tab, tabMeta.name);
                if (try tabs.addTabLabel(shown == tab, tabMeta.name)) {
                    ctx.switchTab(tab);
                }
            }
        }

        var vbox = try dvui.box(@src(), .vertical, .{ .expand = .both, });
        defer vbox.deinit();

        switch (shown) {
            .info   => try tabInfo(ctx),
            .detail => try tabDetail(ctx),
            else    => try dvui.labelNoFmt(@src(), "NOT IMPLEMENTED YET :(", .{
                .gravity_x = 0.5,
                .gravity_y = 0.5,
            }),
        }
    }
}

fn tabInfo(ctx: anytype) !void {
    _ = ctx;

    var vbox = try dvui.box(@src(), .vertical, .{ .expand = .both, });
    defer vbox.deinit();

    const fields = [_][:0]const u8 {"foo", "baz", "bar"};
    for (0.., fields) |idx, name| {
        var hbox = try dvui.box(@src(), .horizontal, .{
            .id_extra = idx,
            .expand   = .horizontal,
        });
        defer hbox.deinit();

        {
            var box = try dvui.box(@src(), .horizontal, .{
                .min_size_content = .{ .w = 64, .h = 0, },
            });
            defer box.deinit();
            try dvui.label(@src(), "{s}", .{name}, .{
                .gravity_x = 1,
            });
        }
        {
            var box = try dvui.box(@src(), .horizontal, .{
                .expand = .horizontal,
            });
            defer box.deinit();
            try dvui.labelNoFmt(@src(), "this is a field value", .{});
        }
    }
}

fn tabDetail(ctx: anytype) !void {
    _ = ctx;
    // const task = ctx.task();

    var vbox = try dvui.box(@src(), .vertical, .{ .expand = .both, });
    defer vbox.deinit();

    // detail
    {
        var detail = try dvui.textEntry(@src(), .{
            .break_lines       = true,
            .scroll_vertical   = true,
            .scroll_horizontal = true,
            .multiline         = true,
        }, .{
            .expand = .both,
        });
        defer detail.deinit();
    }
}

pub const Mock = struct {
    const Self = @This();

    _taskId: usize,
    _summary: []const u8,

    _shownTab: Tab,

    pub fn init(alloc: std.mem.Allocator) !Self {
        _ = alloc;
        return Mock {
            ._taskId = 0,
            ._summary = "hello",
            ._shownTab = .info,
        };
    }
    pub fn deinit(self: *Self) void {
        _ = self;
    }

    fn switchTab(self: *Self, tab: Tab) void {
        self._shownTab = tab;
    }

    fn taskId(self: *const Self) usize { return self._taskId; }
    fn summary(self: *const Self) []const u8 { return self._summary; }

    fn shownTab(self: *const Self) Tab { return self._shownTab; }
};

