const std = @import("std");

const Self = @This();

///
alloc: std.mem.Allocator,

///
id: usize,

///
summary: [:0]const u8,

///
details: [:0]const u8,

///
done: bool,

///
archived: bool,

///
pub fn init(alloc: std.mem.Allocator, id: usize) !Self {
    return Self {
        .alloc    = alloc,
        .id       = id,
        .summary  = try alloc.dupeZ(u8, ""),
        .details  = try alloc.dupeZ(u8, ""),
        .done     = true,
        .archived = true,
    };
}
///
pub fn deinit(self: *Self) void {
    self.alloc.free(self.details);
    self.alloc.free(self.summary);
}

///
pub fn setSummary(self: *Self, v: []const u8) !void {
    self.alloc.free(self.summary);
    self.summary = try self.alloc.dupeZ(u8, v);
}
///
pub fn setDetails(self: *Self, v: []const u8) !void {
    self.alloc.free(self.details);
    self.details = try self.alloc.dupeZ(u8, v);
}

test {
    var task = try init(std.testing.allocator, 0);
    defer task.deinit();
    try std.testing.expectEqual(task.id, 0);
    try std.testing.expectEqualStrings("", task.summary);
    try std.testing.expectEqualStrings("", task.details);

    try task.setSummary("helloworld");
    try std.testing.expectEqualStrings("helloworld", task.summary);
    try std.testing.expectEqualStrings("", task.details);

    try task.setDetails("good afternoon");
    try std.testing.expectEqualStrings("helloworld", task.summary);
    try std.testing.expectEqualStrings("good afternoon", task.details);

    try task.setSummary("goodbye");
    try std.testing.expectEqualStrings("goodbye", task.summary);
    try std.testing.expectEqualStrings("good afternoon", task.details);
}
