const std = @import("std");

const Task = @import("./Task.zig");

const Self = @This();
const Map  = std.AutoHashMap(usize, *Task);

alloc: std.mem.Allocator,
map: Map,
nextId: usize,

///
pub fn init(alloc: std.mem.Allocator) Self {
    return Self {
        .alloc  = alloc,
        .map    = Map.init(alloc),
        .nextId = 0,
    };
}
///
pub fn deinit(self: *Self) void {
    var itr = self.map.valueIterator();
    while (itr.next()) |task| {
        task.*.deinit();
        self.alloc.destroy(task.*);
    }
    self.map.deinit();
}
///
pub fn add(self: *Self) !*Task {
    var task = try self.alloc.create(Task);
    errdefer self.alloc.destroy(task);

    task.* = try Task.init(self.alloc, self.nextId);
    errdefer task.deinit();

    try self.map.putNoClobber(task.id, task);

    self.nextId += 1;
    return task;
}
///
pub fn query(self: *const Self, id: usize) !*Task {
    return self.map.get(id) orelse error.NotFound;
}

test {
    var store = init(std.testing.allocator);
    defer store.deinit();

    const task1 = try store.add();
    const task2 = try store.add();

    try std.testing.expect(task1.id != task2.id);

    try std.testing.expectEqual(task1, store.query(task1.id));
    try std.testing.expectEqual(task2, store.query(task2.id));
    try std.testing.expectError(error.NotFound, store.query(1234));
}
