const std = @import("std");

const Interface = @import("./Interface.zig");

const Task = @import("../Task.zig");

///
const SetSummary = struct {
    ///
    const Self = @This();

    ///
    target: *Task,

    ///
    summary: [:0]const u8,

    ///
    pub fn init(alloc: std.mem.Allocator, task: *Task, summary: []const u8) !Interface {
        const summaryDup = try alloc.dupeZ(u8, summary);
        errdefer alloc.free(summaryDup);
        return try Interface.make(alloc, Self {
            .target  = task,
            .summary = summaryDup,
        });
    }

    ///
    pub fn deinit(self: *Self, alloc: std.mem.Allocator) void {
        alloc.free(self.summary);
    }

    ///
    pub fn apply(self: *Self, alloc: std.mem.Allocator) Interface.Error!void {
        const old = try alloc.dupeZ(u8, self.target.summary);
        errdefer alloc.free(old);

        try self.target.setSummary(self.summary);
        alloc.free(self.summary);

        self.summary = old;
    }

    ///
    pub fn revert(self: *Self, alloc: std.mem.Allocator) Interface.Error!void {
        return self.apply(alloc);
    }
};

test {
    const TaskStore = @import("../TaskStore.zig");

    var tasks = TaskStore.init(std.testing.allocator);
    defer tasks.deinit();

    var task = try tasks.add();
    try task.setSummary("helloworld");

    var sut = try SetSummary.init(std.testing.allocator, task, "goodbye");
    defer sut.deinit();

    try sut.apply();
    try std.testing.expectEqualStrings("goodbye", task.summary);

    try sut.revert();
    try std.testing.expectEqualStrings("helloworld", task.summary);
}
